@@ -1065,6 +1065,83 @@ luck_adj <- function(prevsurv,cursurv,luck,condq=TRUE){
   return(adj_luck)
 }
 
+#' Adjusted Value Calculation
+#'
+#' This function calculates an adjusted value over a time interval with optional discounting.
+#' This is useful for instances when adding cycles may not be desirable, so one can perform
+#' "cycle-like" calculations without needing cycles, offering performance speeds. See
+#' the vignette on avoiding cycles for an example in a model.
+#'
+#' @param curtime Numeric. The current time point.
+#' @param nexttime Numeric. The next time point. Must be greater than or equal to `curtime`.
+#' @param by Numeric. The step size for evaluation within the interval.
+#' @param expression An expression evaluated at each step. Use `time` as the variable within the expression.
+#' @param discount Numeric or NULL. The discount rate to apply, or NULL for no discounting.
+#' 
+#' @export
+#'
+#' @return Numeric. The calculated adjusted value.
+#' 
+#' @importFrom stats weighted.mean
+#' @importFrom utils head
+#' 
+#' @details
+#'  The user can use the `.time` variable to select the corresponding time of the sequence being evaluated.
+#'  For example, in c`urtime = 0, nexttime = 4, by = 1`, `time` would correspond to `0, 1, 2, 3`.
+#'  If using `nexttime = 4.2`, `0, 1, 2, 3, 4`
+#' 
+#'
+#' @examples
+#' # Define a function or vector to evaluate
+#' bs_age <- 1
+#' vec <- 1:8/10
+#' 
+#' # Calculate adjusted value without discounting
+#' adj_val(0, 4, by = 1, expression = vec[floor(.time + bs_age)])
+#' adj_val(0, 4, by = 1, expression = .time * 1.1)
+#'
+#' # Calculate adjusted value with discounting
+#' adj_val(0, 4, by = 1, expression = vec[floor(.time + bs_age)], discount = 0.03)
+#' 
+adj_val <- function(curtime, nexttime, by, expression, discount = NULL) {
+  duration <- nexttime - curtime
+  if (duration < 0) stop("curtime - nexttime is negative (negative duration)")
+  if (duration == 0) return(0)
+  if (!is.null(discount) && is.infinite(discount)) return(0)
+  
+  n_steps <- floor(duration / by)
+  times <- curtime + by * seq(0, n_steps)
+  if (length(times) == 0 || tail(times, 1) < nexttime) {
+    times <- c(times, nexttime)
+  }
+  
+  intervals <- diff(times)
+  eval_times <- head(times, -1)
+  
+  parent <- parent.frame()
+  expr_sub <- substitute(expression)
+  
+  values <- vapply(eval_times, function(tt) {
+    val <- eval(expr_sub, envir = list(.time = tt), enclos = parent)
+    if (is.na(val)) stop("NA value encountered during evaluation at time: ", tt)
+    val
+  }, numeric(1))
+  
+  if (!is.null(discount)) {
+    # Compute present value of $1 over each interval (weights)
+    weights <- disc_ongoing_v(
+      lcldr = discount,
+      lclprvtime = eval_times,
+      lclcurtime = times[-1],
+      lclval = rep(1, length(eval_times))
+    )
+    return(weighted.mean(values, w = weights))
+  } else {
+    return(sum(values * intervals) / duration)
+  }
+}
+
+
 # Continuous and instantaneous discounting ----------------------------------------------------------------------------------------------------------------
 
 #' Calculate discounted costs and qalys between events
@@ -1321,12 +1398,12 @@ disc_cycle_v <- function(
   }
   
   # Compute total possible cycles in this step
-  total_cycles <- pmax(0, ceiling((lclcurtime - starttime) / cyclelength))
-  prev_cycles <-  pmax(0, ceiling((lclprvtime - starttime) / cyclelength))
+  total_cycles <- pmax(0, ceiling((lclcurtime - starttime) / cyclelength),na.rm=TRUE)
+  prev_cycles <-  pmax(0, ceiling((lclprvtime - starttime) / cyclelength),na.rm=TRUE)
   n_cycles <- total_cycles - prev_cycles
   
   # Adjust n_cycles to account for the starting condition
-  n_cycles <- ifelse(lclprvtime == 0 & lclcurtime == 0, 1, n_cycles)
+  n_cycles <- ifelse(lclprvtime == lclcurtime & (lclcurtime - starttime) %% cyclelength == 0, n_cycles+1, n_cycles)
   
   # Cap using max_cycles
   remaining_cycles <- ifelse(is.na(max_cycles), n_cycles, pmax(0, max_cycles - prev_cycles))
@@ -1334,7 +1411,7 @@ disc_cycle_v <- function(
   
   # Compute discount factor per row
   s <- (1 + lcldr)^cyclelength - 1
-  d <- ifelse(lclprvtime==0,-1,lclprvtime / cyclelength)
+  d <- ifelse(lclprvtime==0,0,lclprvtime / cyclelength)-1
   
   discounted <- numeric(n)
   
