@@ -100,7 +100,12 @@ test_that("Vectorial discounting working as expected with vectors", {
   
   #Cycle
   expect_equal(disc_cycle_v(lcldr=0.035, lclcurtime=c(3,3,3), lclval=c(0,1000,Inf),lclprvtime=c(0.5,0.5,0.5), cyclelength=c(1/12,1/12,1/12),starttime=c(0,0,0)),
-               c(0,28215.4394,Inf))  
+               c(0,28296.443022132232727,Inf))  
+  
+  expect_equal(disc_cycle_v(0.035,0,1,1,1000,0,5),1000)
+  expect_equal(disc_cycle_v(0.035,1,1,1,1000,0,5),966.1835748792273)
+  expect_equal(disc_cycle_v(0.035,0,1,0,1000,0,5),1000)
+  expect_equal(disc_cycle_v(0.035,0,1,2,1000,0,5),1966.1835748792273)
 })
 
 test_that("Create indicators works correctly",{
@@ -688,3 +693,344 @@ test_that("generate_stream changes stream length", {
   expect_equal(length(random_stream_instance$stream), 20, info = "generate_stream should change stream to the correct size")
 })
 
+
+# Time covariate tte ------------------------------------------------------
+library(flexsurv)
+
+test_that("qtimecov returns numeric scalar within bounds", {
+  param_fun_factory <- function(p0, p1, p2, p3) {
+    function(t) p0 + p1 * t + p2 * t^2 + p3 * (floor(t) + 1)
+  }
+  
+  rate_exp <- param_fun_factory(0.1, 0, 0, 0)
+  set.seed(1)
+  tte <- qtimecov(runif(1), a_fun = rate_exp, dist = "exp")
+  expect_type(tte, "double")
+  expect_length(tte, 1)
+  expect_gt(tte, 0)
+  expect_lt(tte, 100)
+})
+
+test_that("qtimecov works for all supported distributions", {
+  set.seed(1)
+  param_fun_factory <- function(p0, p1, p2, p3) {
+    function(.time) p0 + p1 * .time + p2 * .time^2 + p3 * (floor(.time) + 1)
+  }
+  
+  # 1. Exponential
+  rate_exp <- param_fun_factory(0.1, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = rate_exp, dist = "exp"))
+  expect_equal(qtimecov(luck = 0.5,a_fun = rate_exp,dist = "exp", dt = 0.001
+  ),qexp(0.5,0.1), tolerance = 0.01)
+  
+  # 2. Gamma
+  shape <- param_fun_factory(2, 0, 0, 0)
+  rate <- param_fun_factory(0.2, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = rate, dist = "gamma"))
+  expect_equal(qtimecov(luck = 0.5,a_fun = shape, b_fun = rate, dist = "gamma", dt = 0.001
+  ),qgamma(0.5,2,0.2), tolerance = 0.01)
+  
+  # 3. Lognormal
+  meanlog <- param_fun_factory(log(10) - 0.5^2 / 2, 0, 0, 0)
+  sdlog <- param_fun_factory(0.5, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = meanlog, b_fun = sdlog, dist = "lnorm"))
+  expect_equal(qtimecov(0.5, a_fun = meanlog, b_fun = sdlog, dist = "lnorm",dt=0.01),
+               qlnorm(0.5,log(10) - 0.5^2 / 2,0.5), tolerance = 0.01)
+  
+  # 4. Normal
+  mean <- param_fun_factory(10, 0, 0, 0)
+  sd <- param_fun_factory(2, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = mean, b_fun = sd, dist = "norm"))
+  expect_equal(qtimecov(0.5, a_fun = mean, b_fun = sd, dist = "norm",dt=0.01),
+               qnorm(0.5,10,2), tolerance = 0.01)
+  
+  # 5. Weibull
+  shape <- param_fun_factory(2, 0, 0, 0)
+  scale <- param_fun_factory(10, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "weibull"))
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibull",dt=0.01),
+               qweibull(0.5,2,10), tolerance = 0.01)
+  
+  # 5.1 WeibullPH
+  shape <- param_fun_factory(2, 0, 0, 0)
+  scale <- param_fun_factory(0.01, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "weibullPH"))
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "weibullPH",dt=0.01),
+               flexsurv::qweibullPH(0.5,2,0.01), tolerance = 0.01)
+  
+  # 6. Loglogistic
+  shape <- param_fun_factory(2.5, 0, 0, 0)
+  scale <- param_fun_factory(7.6, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = scale, dist = "llogis"))
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = scale, dist = "llogis",dt=0.01),
+               flexsurv::qllogis(0.5,2.5,7.6), tolerance = 0.01)
+  
+  # 7. Gompertz
+  shape <- param_fun_factory(0.01, 0, 0, 0)
+  rate <- param_fun_factory(0.091, 0, 0, 0)
+  expect_silent(qtimecov(runif(1), a_fun = shape, b_fun = rate, dist = "gompertz"))
+  expect_equal(qtimecov(0.5, a_fun = shape, b_fun = rate, dist = "gompertz",dt=0.01),
+               qgompertz(0.5,0.01,0.091), tolerance = 0.01)
+  
+  rate_exp <- function(t) 0.1
+  init_luck <- 0.95
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 10)
+    
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    tolerance = 0.01)
+  
+  init_luck <- 0.99
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 5)
+    
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    tolerance = 0.01)
+  
+  
+  init_luck <- 0.3
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001),{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.001, return_luck = TRUE,max_time = 1)
+    
+    qtimecov(luck = a$luck,a_fun = rate_exp,dist = "exp", dt = 0.001, start_time=a$tte)},
+    tolerance = 0.01)
+  
+  
+  
+  init_luck <- 0.3
+  expect_equal(qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000 ,dist = "weibull", dt = 0.001),{
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001, return_luck = TRUE,max_time = 1)
+    
+    qtimecov(luck = a$luck,a_fun = rate_exp,b_fun = function(t) 90000,dist = "weibull", dt = 0.001, start_time=a$tte)},
+    tolerance = 0.01)
+  
+  rate_exp <- function(.time) 0.1
+  rate_exp2 <- function(.time) 0.2
+  time_change <- 10
+  init_luck <- 0.95
+  
+  expect_equal({
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change, return_luck = TRUE)
+    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
+    
+  },{
+    new_luck <- luck_adj(prevsurv = 1 - pexp(q=time_change,rate_exp(1)),
+                         cursurv = 1 - pexp(q=time_change,rate_exp2(1)),
+                         luck = init_luck,
+                         condq = FALSE) #time 10 change
+    qexp(new_luck,rate_exp2(1))
+  }, tolerance = 0.01)
+  
+  
+
+# time varying and an event -----------------------------------------------
+
+  rate_exp <- function(.time) 0.1 + 0.01*.time * 0.00001*.time^2
+  rate_exp2 <- function(.time) 0.2 + 0.02*.time
+  time_change <- 8
+  init_luck <- 0.95
+  
+  expect_equal({
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 0.005, max_time = time_change, return_luck = TRUE)
+    qtimecov(luck = a$luck,a_fun = rate_exp2,dist = "exp", dt = 0.005, start_time=a$tte)
+    
+  },{# Manually reproduce what qtimecov does from a$tte
+    t <- 0
+    luck <- init_luck
+    dt <- 0.005
+    repeat {
+      t <- t + dt
+      s_prev <- 1 - pexp(t - dt, rate = rate_exp(t - dt))
+      s_curr <- 1 - pexp(t,       rate = rate_exp(t - dt))
+      
+      luck <- luck_adj(prevsurv = s_prev, cursurv = s_curr, luck = luck, condq = TRUE)
+      
+      res_tte <- qcond_exp(luck, rate = rate_exp(t))
+      total_tte <- t + res_tte
+      
+      if (res_tte <= dt || total_tte <= t || t >= time_change) {
+        break
+      }
+    }
+    
+    if (total_tte <= time_change) {
+      return(total_tte)
+    }
+    
+    # Phase 2: after change
+    repeat {
+      t <- t + dt
+      s_prev <- 1 - pexp(t - dt, rate = rate_exp2(t - dt))
+      s_curr <- 1 - pexp(t,       rate = rate_exp2(t - dt))
+      
+      luck <- luck_adj(prevsurv = s_prev, cursurv = s_curr, luck = luck, condq = TRUE)
+      
+      res_tte <- qcond_exp(luck, rate = rate_exp2(t))
+      total_tte <- t  + res_tte
+      
+      if (res_tte <= dt || total_tte <= t) {
+        break
+      }
+    }
+    
+    total_tte
+  }, tolerance = 0.01)
+  
+  rate_exp <- function(.time) 0.1 + floor(.time)*0.01
+  init_luck <- 0.95
+  
+  #30x faster 
+  expect_equal({
+    a <- qtimecov(luck = init_luck,a_fun = rate_exp,dist = "exp", dt = 1)
+    a
+  },{
+    time_vec <- 0:100
+    new_luck <- init_luck
+    for (i in 2:length(time_vec)) {
+      new_luck <- luck_adj(prevsurv = 1 - pexp(q=time_vec[i-1],rate_exp(time_vec[i-1])),
+                           cursurv = 1 - pexp(q=time_vec[i],rate_exp(time_vec[i])),
+                           luck = new_luck,
+                           condq = FALSE) 
+      qexp(new_luck,rate_exp(1))
+    }
+    a
+  }, tolerance = 0.01)
+  
+  
+  expect_equal({
+    bs_rate <- 0.08
+    time_cov <- 0.0002271
+    bs_rate_f <- function(.time) bs_rate + time_cov * floor(.time)
+    luck <- 0.403
+    t <- 0
+    
+    # First rate and quantile (from time 0)
+    new_rate <- bs_rate_f(0)
+    ttdeath <- qexp(luck, rate = new_rate)
+    
+    # Unconditional quantile loop
+    while (t < ttdeath) {
+      old_rate <- new_rate
+      new_rate <- bs_rate_f(t)
+      
+      # Both survival values are evaluated at time t (NOT t+1)
+      prev_surv <- 1 - pexp(q = t, rate = old_rate)
+      cur_surv  <- 1 - pexp(q = t, rate = new_rate)
+      
+      # Use condq = FALSE for unconditional approach
+      luck <- luck_adj(prevsurv = prev_surv, cursurv = cur_surv, luck = luck, condq = FALSE)
+      
+      # Unconditional quantile from time 0
+      ttdeath <- qexp(luck, rate = new_rate)
+      
+      t <- t + 1
+    }
+    
+    # Final time-to-event
+    ttdeath
+    
+  },{
+    new_rate <-bs_rate <-  0.08
+    bs_rate_f <- function(.time){bs_rate + (time_cov * floor(.time))}
+    luck <- 0.403
+    time_cov <- 0.0002271
+    t <- 0
+    qtimecov(luck = luck,a_fun = bs_rate_f,dist = "exp", dt = 1)
+  }, tolerance = 0.01)
+  
+})
+
+
+
+test_that("qtimecov throws error for unsupported distribution", {
+  dummy <- function(.time) 1
+  expect_error(qtimecov(runif(1), a_fun = dummy, dist = "beta"), "Unsupported distribution")
+})
+
+test_that("qtimecov respects max_time bound", {
+  slow_fun <- function(.time) 0.00001
+  tte <- qtimecov(
+    luck = 0.999,
+    a_fun = slow_fun,
+    dist = "exp",
+    max_time = 10
+  )
+  expect_lte(tte, 10)
+})
+
+
+test_that("adj_val works as intended",{
+  
+  bs_age <- 1
+  vec <- 1 - seq(from =0, to = 0.2, by = 0.02)
+  expect_equal(adj_val(0,0,by=1, vec[floor(.time + bs_age)], discount = Inf), 0)
+  expect_equal(adj_val(0,5,by=1, vec[floor(.time + bs_age)], discount = Inf), 0)
+  expect_error(adj_val(0,20,by=1, vec[floor(.time + bs_age)]))
+  expect_error(adj_val(0,20,by=1, vec[floor(.time + bs_age)]))
+  expect_error(adj_val(0,-5,by=1, vec[floor(.time + bs_age)]))
+  expect_equal(adj_val(0,0,by=1, vec[floor(.time + bs_age)]), 0)
+  expect_equal(adj_val(0,0.1,by=1, vec[floor(.time + bs_age)], discount = 0),1)
+  expect_equal(adj_val(0,1.1,by=1, vec[floor(.time + bs_age)] * .time, discount = 0),(1*0+0.98*0.1)/1.1)
+  expect_equal(adj_val(0,0.1,by=1, vec[floor(.time + bs_age)] * .time, discount = 0),0)
+  expect_equal(adj_val(0,0.1,by=1, vec[floor(.time + bs_age)] * .time, discount = 0.03),0)
+  expect_equal(adj_val(8,9,by=0.2, vec[floor(.time + bs_age)], discount = 0),0.84)
+  expect_equal(adj_val(8,9,by=0.5, vec[floor(.time + bs_age)], discount = Inf),0)
+  expect_equal(adj_val(8,9,by=0.5, 1),1)
+  expect_equal(adj_val(0,4,by=1, .time),1.5)
+  
+  expect_equal({
+    val <- 0.8 * adj_val(0,5.2,by=1, vec[floor(.time + bs_age)], discount = 0.03)
+    disc_ongoing_v(0.03,
+                   0,
+                   5.2,
+                   val)
+  },{
+    a <- 0
+    time_vec <- c(0,1,2,3,4,5,5.2)
+    for (i in 1:(length(time_vec)-1)) {
+      a <-  a + disc_ongoing_v(0.03,
+                               time_vec[i],
+                               time_vec[i+1],
+                               0.8 * vec[floor(time_vec[i] + bs_age)])
+    }
+    a
+  })
+  
+  #17x faster
+  expect_equal({
+    val <- 0.8 * adj_val(0,5.2,by=1, vec[floor(.time + bs_age)], discount = 0)
+    disc_ongoing_v(0,
+                   0,
+                   5.2,
+                   val)
+  },{
+    a <- 0
+    time_vec <- c(0,1,2,3,4,5,5.2)
+    for (i in 1:(length(time_vec)-1)) {
+      a <-  a + disc_ongoing_v(0,
+                               time_vec[i],
+                               time_vec[i+1],
+                               0.8 * vec[floor(time_vec[i] + bs_age)])
+    }
+    a
+  })
+  
+})
+
+test_that("qcondweibull and qcondweibullPH can give same results",
+          {
+            # Set base R parameters
+            shape <- 2
+            scale_base <- 10
+            # Convert to PH parameter
+            scale_PH <- (1 / scale_base)^shape  # = 0.01
+            
+            # Set common inputs
+            p <- 0.5
+            t0 <- 5
+            
+            # Evaluate both
+            expect_equal(qcond_weibull(p, shape, scale_base, lower_bound = t0),
+            qcond_weibullPH(p, shape, scale_PH, lower_bound = t0))
+            }
+          )
